﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using San9.Resource.IO;
using San9.Resource.Message;

namespace San9.Editor.Forms {
  public partial class MessageEditorForm : Form {
    public MessageEditorForm() {
      InitializeComponent();
    }

    private readonly int address;
    public BinaryMessage Result = null;

    public MessageEditorForm(BinaryMessage source) {
      InitializeComponent();
      txtBinary.Text = source.Binary;
      txtBinary.Focus();
      this.address = int.Parse(source.Address, NumberStyles.HexNumber);
    }

    private void btnReflect_Click(object sender, EventArgs e) {
      int start = txtBinary.SelectionStart;
      int length = txtBinary.SelectionLength;
      string pre = start == 0 ? string.Empty : txtBinary.Text.Substring(0, start);
      string suf = length == 0 ? string.Empty : txtBinary.Text.Substring(start + length);
      string body = new BinaryMessage(address, new TextMessage(txtText.Text).Source).Binary;
      int newlength = body.Length;
      txtBinary.Text = pre + body + suf;
      txtBinary.SelectionStart = start;
      txtBinary.SelectionLength = newlength;
      txtBinary.Focus();
      txtText.Text = string.Empty;
      txtText.Enabled = false;
      btnReflect.Enabled = false;
    }

    private void btnOK_Click(object sender, EventArgs e) {
      try {
        Result = new BinaryMessage(address, txtBinary.Text);
        DialogResult = DialogResult.OK;
        Close();
      } catch (Exception ex) {
        MessageBox.Show(ex.Message, "エラー", MessageBoxButtons.OK);
      }
    }

    private void btnCancel_Click(object sender, EventArgs e) {
      DialogResult = DialogResult.Cancel;
      Close();
    }

    private void txtBinary_MouseUp(object sender, MouseEventArgs e) {
      int start = txtBinary.SelectionStart;
      int length = txtBinary.SelectionLength;
      while (length > 0 && isWhiteSpace(txtBinary.Text[start])) {
        start++;
        length--;
      }
      while (start > 0 && !isWhiteSpace(txtBinary.Text[start - 1])) {
        start--;
        length++;
      }
      while (length > 0 && isWhiteSpace(txtBinary.Text[start + length - 1])) {
        length--;
      }
      while (start + length < txtBinary.Text.Length && !isWhiteSpace(txtBinary.Text[start + length])) {
        length++;
      }
      txtBinary.SelectionStart = start;
      txtBinary.SelectionLength = length;
      try {
        txtText.Text = TextMessage.Parse(new ByteReader(toBytes(txtBinary.SelectedText))).Text;
        txtText.Enabled = true;
        btnReflect.Enabled = true;
      } catch {
        txtText.Text = string.Empty;
        txtText.Enabled = false;
        btnReflect.Enabled = false;
      }
    }

    private static byte[] toBytes(string text) {
      string[] s = text.Split(new char[] { ' ', '\n' }, StringSplitOptions.RemoveEmptyEntries);
      byte[] b = new byte[s.Length];
      for (int i = 0; i < b.Length; i++)
        b[i] = byte.Parse(s[i], NumberStyles.HexNumber);
      return b;
    }

    private static bool isWhiteSpace(char c) {
      return c == ' ' || c == '\n';
    }
  }
}
