﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using San9.Resource.Compression;
using San9.Resource.Message;

namespace San9.Editor.Forms {
  public partial class MessageSelectorForm : Form {
    public MessageSelectorForm() {
      InitializeComponent();
    }

    private string filename;

    public MessageSelectorForm(IEnumerable<byte> source, string filename) {
      InitializeComponent();
      this.messageResourceBindingSource.DataSource = new MessageResource(Ls11.Decode(source));
      this.filename = filename;
    }

    private void btnAdd_Click(object sender, EventArgs e) {
      if (MessageBox.Show("メッセージを追加します", "確認", MessageBoxButtons.OKCancel) == DialogResult.OK) {
        messageResourceBindingSource.Add(new BinaryMessage(messageResourceBindingSource.Count, "05 05 05"));
        messageResourceBindingSource.MoveLast();
      }
    }

    private void btnEdit_Click(object sender, EventArgs e) {
      if (messageResourceDataGridView.CurrentCell != null) {
        BinaryMessage message = messageResourceBindingSource.Current as BinaryMessage;
        MessageEditorForm form = new MessageEditorForm(message);
        if (form.ShowDialog() == DialogResult.OK) {
          BinaryMessage result = form.Result;
          int index = int.Parse(result.Address, NumberStyles.HexNumber);
          messageResourceBindingSource.RemoveAt(index);
          messageResourceBindingSource.Insert(index, result);
          messageResourceDataGridView[0, index].Selected = true;
        }
      }
    }

    private void btnSave_Click(object sender, EventArgs e) {
      SaveFileDialog saveFileDialog = new SaveFileDialog();
      saveFileDialog.InitialDirectory = filename;
      saveFileDialog.Filter = "メッセージ ファイル (*.s9)|*.s9|すべてのファイル (*.*)|*.*";
      if (saveFileDialog.ShowDialog(this) == DialogResult.OK) {
        filename = saveFileDialog.FileName;
        byte[] source = Ls11.Encode((messageResourceBindingSource.DataSource as MessageResource).Source).ToArray();
        using (FileStream fs = new FileStream(filename, FileMode.Create, FileAccess.Write)) {
          fs.Write(source, 0, source.Length);
        }
      }
    }
  }
}
