﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using San9.Resource.IO;

namespace San9.Resource.Compression {
  public static class Ls11 {
    static readonly byte[] Identifier = new byte[] { 0x4C, 0x53, 0x31, 0x31, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
    static readonly byte[] Dictionary = new byte[] { 0x00, 0x0A, 0x90, 0xC9, 0x8E, 0x8C, 0xCA, 0x91, 0x8F, 0x93, 0x97, 0xA6, 0xC6, 0x96, 0x95, 0x8B, 0x89, 0x92, 0xDE, 0x8D, 0x8A, 0xC4, 0xB2, 0xC0, 0xB6, 0x94, 0xD3, 0xBC, 0xC5, 0xC3, 0xB8, 0x81, 0xB3, 0xBD, 0xD7, 0xBA, 0x88, 0xD9, 0xB7, 0xD8, 0xDA, 0x02, 0xCF, 0xD4, 0x41, 0x01, 0xBF, 0x98, 0xD6, 0xBE, 0xB1, 0xB5, 0xBB, 0xAF, 0xDD, 0xB4, 0xC1, 0xD2, 0xCD, 0xB9, 0xC2, 0xDC, 0x04, 0x82, 0xF0, 0x03, 0x05, 0xC7, 0x07, 0x4A, 0xCE, 0xE3, 0x06, 0xCC, 0xE0, 0xE5, 0x53, 0x69, 0x76, 0x08, 0xCB, 0x09, 0x42, 0xAC, 0x9F, 0xA9, 0xD0, 0xDB, 0xAB, 0x4F, 0xE8, 0xED, 0x47, 0xE6, 0x45, 0x73, 0x9D, 0x52, 0xC8, 0xAD, 0x6C, 0x9E, 0xA8, 0x40, 0x87, 0x6E, 0x1B, 0x43, 0x34, 0xA7, 0xAA, 0xAE, 0xEA, 0xFB, 0x5F, 0x84, 0xD1, 0x75, 0xEC, 0x49, 0xF6, 0xDF, 0xE9, 0x79, 0xF1, 0xE7, 0x48, 0x62, 0x6A, 0x6F, 0x61, 0x5C, 0x60, 0x78, 0x7A, 0xA2, 0xF3, 0x77, 0x32, 0xFC, 0x0B, 0x55, 0x6B, 0x71, 0x83, 0xA1, 0x31, 0x3D, 0x63, 0x65, 0x70, 0x7B, 0xD5, 0x4B, 0x4E, 0x86, 0xEF, 0x51, 0x67, 0xF5, 0x56, 0x44, 0xE1, 0xE4, 0x33, 0xE2, 0x4D, 0x9B, 0xFA, 0xF8, 0x4C, 0x58, 0x5D, 0x50, 0x7E, 0x54, 0x5E, 0x6D, 0x80, 0x99, 0xA4, 0x46, 0x9C, 0x30, 0xF7, 0x68, 0x64, 0x66, 0xF4, 0x74, 0xA0, 0xB0, 0x36, 0x7C, 0xEE, 0x57, 0x9A, 0x35, 0x37, 0x5A, 0x59, 0x5B, 0x7D, 0x85, 0xEB, 0xF2, 0x39, 0x72, 0xA3, 0x3E, 0xA5, 0x1F, 0x20, 0x2B, 0xF9, 0x3F, 0x21, 0xFF, 0x24, 0x38, 0x13, 0x28, 0x1E, 0x29, 0x12, 0x1C, 0x2E, 0x11, 0x1A, 0x2C, 0x15, 0x25, 0x7F, 0x2F, 0x16, 0x3B, 0x3C, 0x2D, 0x0C, 0x27, 0xFD, 0x1D, 0x2A, 0x0F, 0x10, 0x26, 0x18, 0x14, 0x17, 0x23, 0x0D, 0xFE, 0x19, 0x22, 0x3A, 0x0E };
    static readonly int MinRepeatLength = 3;
    static readonly int BodyAddress = 0x120;

    #region Encode
    public static IEnumerable<byte> Encode(IEnumerable<byte> source) {
      Dictionary<byte, int> dictionary = convertForEncoding(Dictionary);
      Dictionary<int, int> hisotry = new Dictionary<int, int>();
      ByteReader reader = new ByteReader(source);
      BitWriter writer = new BitWriter(reader.Rest * 2);
      while (!reader.AtEnd) {
        int index = reader.Index;
        int digest = getDigest(reader);
        if (!hisotry.ContainsKey(digest)) {
          writer.Append(toBitArray(dictionary[reader.Current]));
          reader.Skip(1);
        } else {
          int distance = index - hisotry[digest];
          int length = MinRepeatLength;
          while (length < reader.Rest && reader[reader.Index + length] == reader[reader.Index + length - distance]) length++;
          writer.Append(toBitArray(distance + Dictionary.Length));
          writer.Append(toBitArray(length - MinRepeatLength));
          reader.Skip(length);
          hisotry.Remove(digest);
        }
        if (digest >= 0) {
          hisotry.Add(digest, index);
        }
      }
      byte[] body = writer.Result.ToArray();
      ByteWriter result = new ByteWriter(Identifier.Length + Dictionary.Length + 16 + body.Length);
      result.Append(Identifier);
      result.Append(Dictionary);
      result.IntOnBigEndian(body.Length);
      result.IntOnBigEndian(source.ToArray().Length);
      result.IntOnBigEndian(BodyAddress);
      result.IntOnBigEndian(0);
      result.Append(body);
      return result.Result;
    }

    private static Dictionary<byte, int> convertForEncoding(byte[] dictionary) {
      Dictionary<byte, int> result = new Dictionary<byte, int>();
      for (int i = 0; i < dictionary.Length; i++)
        result.Add(dictionary[i], i);
      return result;
    }
    private static int getDigest(ByteReader reader) {
      if (reader.Rest < MinRepeatLength) return -1;
      return (reader[reader.Index] << 16) + (reader[reader.Index + 1] << 8) + reader[reader.Index + 2]; 
    }
    private static IEnumerable<bool> toBitArray(int code) {
      int length = getCodeLength(code);
      int pre = (1 << length) - 2;
      int suf = code - pre;
      for (int i = length - 1; i >= 0; i--)
        yield return ((pre >> i) & 1) != 0;
      for (int i = length - 1; i >= 0; i--)
        yield return ((suf >> i) & 1) != 0;
    }
    private static int getCodeLength(int code) {
      for (int length = 1; length < 32; length++)
        if (((2 << length) - 3) >= code)
          return length;
      throw new OverflowException();
    }
    #endregion

    #region Decode
    public static IEnumerable<byte> Decode(IEnumerable<byte> source) {
      ByteReader reader = new ByteReader(source);
      if (!checkIdentifier(reader)) throw new ArgumentException();
      byte[] dictionary = reader.Take(Dictionary.Length).ToArray();
      reader.Skip(Dictionary.Length);
      int encodeSize = reader.IntOnBigEndian();
      int originSize = reader.IntOnBigEndian();
      int bodyAddress = reader.IntOnBigEndian();
      reader.Skip(bodyAddress - reader.Index);
      BitReader body = new BitReader(reader.Take(encodeSize).ToArray());
      ByteWriter writer = new ByteWriter(originSize);
      while (!body.AtEnd && !writer.IsFull) {
        int code1 = read(body);
        if (code1 < Dictionary.Length) {
          writer.Append(dictionary[code1]);
        } else {
          int code2 = read(body);
          int distance = code1 - Dictionary.Length;
          int length = code2 + MinRepeatLength;
          for (int i = 0; i < length; i++)
            writer.Append(writer[writer.Index - distance]);
        }
      }
      if (!writer.IsFull) throw new ArgumentException();
      return writer.Result;
    }

    private static bool checkIdentifier(ByteReader reader) {
      byte[] identifier = reader.Take(Identifier.Length).ToArray();
      for (int i = 0; i < identifier.Length; i++)
        if (identifier[i] != Identifier[i])
          return false;
      reader.Skip(identifier.Length);
      return true;
    }
    private static int read(BitReader body) {
      bool[] pre = body.TakeUntil(b => !b).ToArray();
      body.Skip(pre.Length);
      bool[] suf = body.Take(pre.Length).ToArray();
      body.Skip(suf.Length);
      return toInt(pre) + toInt(suf);
    }
    private static int toInt(IEnumerable<bool> bits) {
      int val = 0;
      foreach (bool b in bits)
        val = (val << 1) + (b ? 1 : 0);
      return val;
    }
    #endregion
  }
}
