﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace San9.Resource.IO {
  public class BitWriter {
    private readonly byte[] destination;
    private int index = 0;

    public BitWriter(int n) {
      this.destination = new byte[n];
    }

    public int Index {
      get { return index; }
    }

    public byte this[int i] {
      get {
        if (i < 0 || ((index + 7) / 8) <= i) throw new ArgumentOutOfRangeException();
        return destination[i];
      }
    }

    public void Append(bool b) {
      if (IsFull) throw new OverflowException();
      if (b) destination[index / 8] = (byte)(destination[index / 8] | (0x80 >> (index % 8)));
      index++;
    }

    public void Append(IEnumerable<bool> bs) {
      foreach (bool b in bs) Append(b);
    }

    public IEnumerable<byte> Result {
      get {
        int end = (index + 7) / 8;
        for (int i = 0; i < end; i++)
          yield return destination[i];
      }
    }

    public bool IsFull {
      get { return index == destination.Length * 8; }
    }
  }
}
