﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace San9.Resource.IO {
  public class ByteWriter {
    private readonly byte[] destination;
    private int index = 0;

    public ByteWriter(int n) {
      this.destination = new byte[n];
    }

    public int Index {
      get { return index; }
    }

    public byte this[int i] {
      get {
        if (i < 0 || index <= i) throw new ArgumentOutOfRangeException();
        return destination[i];
      }
    }

    public void Append(byte b) {
      if (index == destination.Length) throw new OverflowException();
      destination[index] = b;
      index++;
    }

    public void Append(IEnumerable<byte> bs) {
      foreach (byte b in bs) Append(b);
    }

    public void ShortOnBigEndian(short s) {
      Append((byte)((s >> 8) & 0xFF));
      Append((byte)(s & 0xFF));
    }

    public void ShortOnLittleEndian(short s) {
      Append((byte)(s & 0xFF));
      Append((byte)((s >> 8) & 0xFF));
    }

    public void IntOnBigEndian(int i) {
      Append((byte)((i >> 24) & 0xFF));
      Append((byte)((i >> 16) & 0xFF));
      Append((byte)((i >> 8) & 0xFF));
      Append((byte)(i & 0xFF));
    }

    public void IntOnLittleEndian(int i) {
      Append((byte)(i & 0xFF));
      Append((byte)((i >> 8) & 0xFF));
      Append((byte)((i >> 16) & 0xFF));
      Append((byte)((i >> 24) & 0xFF));
    }

    public IEnumerable<byte> Result {
      get {
        int end = index;
        for (int i = 0; i < end; i++)
          yield return destination[i];
      }
    }

    public bool IsFull {
      get { return index == destination.Length; }
    }
  }
}
