﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace San9.Resource.IO {
  public abstract class Riff {
    private static readonly byte[] Identifier = Encoding.ASCII.GetBytes("RIFF");

    public abstract string FormType {
      get;
    }

    private List<Chunk> chunks = new List<Chunk>();

    public Riff(IEnumerable<byte> source) {
      ByteReader reader = new ByteReader(source);
      if (!check(reader, Identifier)) throw new ArgumentException();
      int size = reader.IntOnLittleEndian();
      ByteReader data = reader.Take(size);
      if (Encoding.ASCII.GetString(data.Take(4).ToArray()) != FormType) throw new ArgumentException();
      data.Skip(4);
      while (!data.AtEnd) {
        string id = Encoding.ASCII.GetString(data.Take(4).ToArray());
        data.Skip(4);
        int sz = data.IntOnLittleEndian();
        byte[] dt = data.Take(sz).ToArray();
        chunks.Add(new Chunk(id, dt));
        data.Skip(dt.Length);
      }
    }

    protected static bool check(ByteReader source, byte[] expected) {
      for (int i = 0; i < expected.Length; i++)
        if (expected[i] != source[source.Index + i])
          return false;
      source.Skip(expected.Length);
      return true;
    }

    public IEnumerable<byte> Source {
      get {
        int length = 4;
        foreach (Chunk c in chunks)
          length += c.Length;
        ByteWriter writer = new ByteWriter(8 + length);
        writer.Append(Identifier);
        writer.IntOnLittleEndian(length);
        writer.Append(Encoding.ASCII.GetBytes(FormType));
        foreach (Chunk c in chunks)
          writer.Append(c.Source);
        return writer.Result;
      }
    }
  }
}
