﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;

namespace San9.Resource.Message {
  public class BinaryMessage : Message {
    private static readonly Encoding encoding = Encoding.GetEncoding(932);

    private readonly int address;
    private readonly byte[] source;

    public BinaryMessage(int address, IEnumerable<byte> source) {
      this.address = address;
      this.source = source.ToArray();
    }

    public BinaryMessage(int address, string binary) {
      string[] bytes = binary.Split(new char[] { ' ', '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);
      byte[] source = new byte[bytes.Length];
      try {
        for (int i = 0; i < source.Length; i++)
          source[i] = byte.Parse(bytes[i], NumberStyles.HexNumber);
      } catch {
        throw new ArgumentException("バイト列にパースできません");
      }
      try {
        Message message = Message.Parse(source);
      } catch {
        throw new ArgumentException("メッセージにパースできません");
      }
      this.address = address;
      this.source = source;
    }

    public override IEnumerable<byte> Source {
      get {
        foreach (byte b in source)
          yield return b;
      }
    }

    public int Length {
      get { return source.Length; }
    }

    public string Address {
      get { return address.ToString("X4"); }
    }

    public string Binary {
      get {
        StringBuilder bld = new StringBuilder();
        foreach (byte b in source)
          bld.Append(b.ToString("X2") + " ");
        return bld.ToString().Trim().Replace(" 05 05 04", "\n05 05 04").Replace("05 05 05 ", "05 05 05\n");
      }
    }

    public string Text {
      get {
        return encoding.GetString(source);
      }
    }
  }
}
