﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using San9.Resource.IO;

namespace San9.Resource.Message {
  public class BranchMessage : Message {
    private static readonly byte[] Prefix = new byte[] { 0x05, 0x05, 0x04 };

    private readonly Condition condition;
    private readonly Message trueCase;
    private readonly Message falseCase;

    public BranchMessage(Condition condition, Message trueCase, Message falseCase) {
      this.condition = condition;
      this.trueCase = trueCase;
      this.falseCase = falseCase;
    }

    public static BranchMessage Parse(ByteReader source) {
      if (!check(source, Prefix)) return null;
      Condition condition = Condition.Parse(source);
      Message trueCase = readTrueCase(source);
      Message falseCase = readFalseCase(source);
      return new BranchMessage(condition, trueCase, falseCase);
    }

    private static Message readTrueCase(ByteReader source) {
      Message message = BranchMessage.Parse(source);
      if (message != null) return message;
      message = JumpMessage.Parse(source);
      if (message != null) return message;
      message = TextMessage.Parse(source);
      return message;
    }

    private static Message readFalseCase(ByteReader source) {
      Message message = BranchMessage.Parse(source);
      if (message != null) return message;
      message = JumpMessage.Parse(source);
      if (message != null) return message;
      message = RandomMessage.Parse(source);
      if (message != null) return message;
      message = TextMessage.Parse(source);
      return message;
    }

    public override IEnumerable<byte> Source {
      get {
        foreach (byte b in Prefix)
          yield return b;
        foreach (byte b in condition.Source)
          yield return b;
        foreach (byte b in trueCase.Source)
          yield return b;
        foreach (byte b in falseCase.Source)
          yield return b;
      }
    }
  }
}
