﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using San9.Resource.IO;
using San9.Resource.Utility;

namespace San9.Resource.Message {
  public class Condition {
    private static readonly byte OperaterPrefix = 0x04;

    public enum Operater {
      Equal, NotEqual, GreaterThan, NotGreaterThan, LessThan, NotLessThan
    }

    private static readonly BiDictionary<Bytes, Operater> dictionary = new BiDictionary<Bytes, Operater>() {
      { new Bytes(0x3D), Operater.Equal },
      { new Bytes(0x3D, 0x25), Operater.NotEqual },
      { new Bytes(0x7D), Operater.GreaterThan },
      { new Bytes(0x7D, 0x25), Operater.NotGreaterThan },
      { new Bytes(0x7B), Operater.LessThan },
      { new Bytes(0x7B, 0x25), Operater.NotLessThan }
    };

    private readonly Operand left;
    private readonly Operater? ope;
    private readonly Operand right;

    public Condition(Operand left, Operater ope, Operand right) {
      this.left = left;
      this.ope = ope;
      this.right = right;
    }

    public Condition(Operand left) {
      this.left = left;
      this.ope = null;
      this.right = null;
    }

    public static Condition Parse(ByteReader source) {
      Operand left = readOperand(source);
      if (!source.AtEnd && source.Current == OperaterPrefix) {
        Operater ope = readOperater(source);
        Operand right = readOperand(source);
        return new Condition(left, ope, right);
      } else {
        return new Condition(left);
      }
    }

    private static Operand readOperand(ByteReader source) {
      Operand o = Noun.Parse(source);
      if (o != null) return o;
      o = Number.Parse(source);
      if (o != null) return o;
      throw new ArgumentException();
    }

    private static Operater readOperater(ByteReader source) {
      if (source.Current != OperaterPrefix) throw new ArgumentException();
      source.Skip(1);
      for (int i = 2; i >= 1; i--) {
        Bytes key = new Bytes(source.Take(i).ToArray());
        if (dictionary.ContainsKey(key)) {
          source.Skip(key.Length);
          return dictionary[key];
        }
      }
      throw new ArgumentException();
    }

    public IEnumerable<byte> Source {
      get {
        foreach (byte b in left.Source)
          yield return b;
        if (ope != null) {
          yield return OperaterPrefix;
          foreach (byte b in dictionary[ope.Value].Source)
            yield return b;
          foreach (byte b in right.Source)
            yield return b;
        }
      }
    }
  }
}