﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using San9.Resource.IO;

namespace San9.Resource.Message {
  public class JumpMessage : Message {
    private static readonly byte[] Prefix = new byte[] { 0x01, 0x4A };
    private static readonly byte[] Suffix = new byte[] { 0x05, 0x05, 0x05 };

    public readonly short Address;

    public JumpMessage(short address) {
      this.Address = address;
    }

    public static JumpMessage Parse(ByteReader source) {
      if (!check(source, Prefix)) return null;
      short address = source.ShortOnLittleEndian();
      if (!check(source, Suffix)) throw new ArgumentException();
      return new JumpMessage(address);
    }

    public override IEnumerable<byte> Source {
      get {
        foreach (byte b in Prefix)
          yield return b;
        yield return (byte)(Address & 0xFF);
        yield return (byte)(Address >> 8);
        foreach (byte b in Suffix)
          yield return b;
      }
    }
  }
}
