﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using San9.Resource.IO;
using San9.Resource.Utility;
using System.ComponentModel;

namespace San9.Resource.Message {
  public class MessageResource : BindingList<BinaryMessage> {
    public MessageResource(IEnumerable<byte> source) {
      ByteReader reader = new ByteReader(source);
      int count = reader.ShortOnLittleEndian();
      int[] addresses = new int[count];
      for (int i = 0; i < addresses.Length; i++)
        addresses[i] = reader.IntOnLittleEndian();
      for (int i = 0; i < addresses.Length - 1; i++) {
        int length = addresses[i + 1] - addresses[i];
        this.Add(new BinaryMessage(i, reader.Take(length).ToArray()));
        reader.Skip(length);
      }
      this.Add(new BinaryMessage(count - 1, reader.Take(reader.Rest - 1).ToArray()));
    }

    public IEnumerable<byte> Source {
      get {
        int[] addresses = new int[this.Count];
        addresses[0] = 2 + 4 * this.Count;
        for (int i = 1; i < this.Count; i++)
          addresses[i] = addresses[i - 1] + this[i - 1].Length;
        ByteWriter writer = new ByteWriter(2000000);
        writer.ShortOnLittleEndian((short)this.Count);
        foreach (int address in addresses)
          writer.IntOnLittleEndian(address);
        for (int i = 0; i < this.Count; i++)
          writer.Append(this[i].Source);
        writer.Append(0);
        return writer.Result;
      }
    }
  }
}
