﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using San9.Resource.IO;

namespace San9.Resource.Message {
  public class Number : Operand {
    public static readonly int MinValue = 0;
    public static readonly int MaxValue = 999;

    public readonly int Value;

    public Number(int value) {
      if (value < MinValue || MaxValue < value) throw new ArgumentOutOfRangeException();
      this.Value = value;
    }

    public static Number Parse(ByteReader source) {
      byte[] bytes = source.TakeWhile(b => 0x30 <= b && b <= 0x39).ToArray();
      if (bytes.Length == 0) return null;
      int value = int.Parse(Encoding.ASCII.GetString(bytes));
      source.Skip(bytes.Length);
      return new Number(value);
    }

    public override IEnumerable<byte> Source {
      get { return Encoding.ASCII.GetBytes(Value.ToString()); }
    }

    public override string Text {
      get { return Value.ToString(); }
    }
  }
}
