﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using San9.Resource.IO;

namespace San9.Resource.Message {
  public class RandomMessage : Message {
    private static readonly byte[] Prefix = new byte[] { 0x01, 0x53, 0x02, 0xCA, 0x25 };

    private readonly Message[] messages;

    public RandomMessage(IEnumerable<Message> messages) {
      this.messages = messages.ToArray();
    }

    public static RandomMessage Parse(ByteReader source) {
      if (!check(source, Prefix)) return null;
      int count = Number.Parse(source).Value;
      Message[] messages = new Message[count];
      for (int i = 0; i < count; i++)
        messages[i] = TextMessage.Parse(source);
      return new RandomMessage(messages);
    }

    public override IEnumerable<byte> Source {
      get {
        foreach (byte b in Prefix)
          yield return b;
        foreach (byte b in new Number(messages.Length).Source)
          yield return b;
        foreach (Message message in messages)
          foreach (byte b in message.Source)
            yield return b;
      }
    }
  }
}
