﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace San9.Resource.Utility {
  public class Bytes : IEquatable<Bytes> {
    private readonly byte[] source;

    public Bytes(params byte[] source) {
      if (source.Length == 0) throw new ArgumentException();
      this.source = new byte[source.Length];
      source.CopyTo(this.source, 0);
    }

    public IEnumerable<byte> Source {
      get {
        foreach (byte b in source)
          yield return b;
      }
    }

    public int Length {
      get { return source.Length; }
    }

    public bool Equals(Bytes that) {
      if (this.source.Length != that.source.Length)
        return false;
      for (int i = 0; i < this.source.Length; i++)
        if (this.source[i] != that.source[i])
          return false;
      return true;
    }

    public override bool Equals(object obj) {
      Bytes that = obj as Bytes;
      if (that == null) return false;
      return Equals(that);
    }

    public override int GetHashCode() {
      if (source.Length > 2)
        return source[0] + (source[1] << 8) + (source[2] << 16);
      else if (source.Length > 1)
        return source[0] + (source[1] << 8);
      else if (source.Length > 0)
        return source[0];
      else
        return 0;
    }
  }
}
