﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;

namespace San9.Resource.IO {
  [TestFixture]
  public class BitReaderTest {
    [Test]
    public void Test1() {
      BitReader reader = new BitReader(new byte[] { 0x12 });
      check(reader, 0, 8, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 1, 7, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 2, 6, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 3, 5, false, true);
      Assert.AreEqual(true, reader.Next());
      check(reader, 4, 4, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 5, 3, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 6, 2, false, true);
      Assert.AreEqual(true, reader.Next());
      check(reader, 7, 1, false, false);
      Assert.AreEqual(false, reader.Next());
      Assert.AreEqual(true, reader.AtEnd);
    }

    [Test]
    public void Test2() {
      BitReader reader = new BitReader(new byte[] { 0x12, 0x34 });
      reader = reader.Take(10).TakeUntil(b => b);
      check(reader, 0, 4, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 1, 3, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 2, 2, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 3, 1, false, true);
      Assert.AreEqual(false, reader.Next());
      Assert.AreEqual(true, reader.AtEnd);
      reader.Reset();
      reader = reader.TakeWhile(b => !b);
      check(reader, 0, 3, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 1, 2, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 2, 1, false, false);
      Assert.AreEqual(false, reader.Next());
      Assert.AreEqual(true, reader.AtEnd);
    }

    [Test]
    public void Test3() {
      BitReader reader = new BitReader(new byte[] { 0x12, 0x34 });
      reader = reader.Drop(2).DropUntil(b => b).DropWhile(b => !b);
      check(reader, 0, 10, false, true);
      Assert.AreEqual(true, reader.Next());
      check(reader, 1, 9, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 2, 8, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 3, 7, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 4, 6, false, true);
      Assert.AreEqual(true, reader.Next());
      check(reader, 5, 5, false, true);
      Assert.AreEqual(true, reader.Next());
      check(reader, 6, 4, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 7, 3, false, true);
      Assert.AreEqual(true, reader.Next());
      check(reader, 8, 2, false, false);
      Assert.AreEqual(true, reader.Next());
      check(reader, 9, 1, false, false);
      Assert.AreEqual(false, reader.Next());
      Assert.AreEqual(true, reader.AtEnd);
    }

    private void check(BitReader reader, int index, int rest, bool atEnd, bool current) {
      Assert.AreEqual(index, reader.Index);
      Assert.AreEqual(rest, reader.Rest);
      Assert.AreEqual(atEnd, reader.AtEnd);
      Assert.AreEqual(current, reader.Current);
    }
  }
}
