﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;

namespace San9.Resource.IO {
  [TestFixture]
  public class ByteReaderTest {
    [Test]
    public void Test1() {
      ByteReader reader = new ByteReader(new byte[] { 1, 2, 3 });
      check(reader, 0, 3, false, 1);
      Assert.AreEqual(true, reader.Next());
      check(reader, 1, 2, false, 2);
      Assert.AreEqual(true, reader.Next());
      check(reader, 2, 1, false, 3);
      Assert.AreEqual(false, reader.Next());
      Assert.AreEqual(true, reader.AtEnd);
      reader.Reset();
      check(reader, 0, 3, false, 1);
      Assert.AreEqual(true, reader.Next());
      check(reader, 1, 2, false, 2);
      Assert.AreEqual(true, reader.Next());
      check(reader, 2, 1, false, 3);
      Assert.AreEqual(false, reader.Next());
      Assert.AreEqual(true, reader.AtEnd);
    }

    [Test]
    public void Test2() {
      ByteReader reader = new ByteReader(new byte[] { 1, 2, 3, 4, 5, 6, 7, 8, 9 });
      reader = reader.Take(5).TakeWhile(b => b < 5).TakeUntil(b => b == 3);
      check(reader, 0, 3, false, 1);
      Assert.AreEqual(true, reader.Next());
      check(reader, 1, 2, false, 2);
      Assert.AreEqual(true, reader.Next());
      check(reader, 2, 1, false, 3);
      Assert.AreEqual(false, reader.Next());
      Assert.AreEqual(true, reader.AtEnd);
      reader.Reset();
      check(reader, 0, 3, false, 1);
      Assert.AreEqual(true, reader.Next());
      check(reader, 1, 2, false, 2);
      Assert.AreEqual(true, reader.Next());
      check(reader, 2, 1, false, 3);
      Assert.AreEqual(false, reader.Next());
      Assert.AreEqual(true, reader.AtEnd);
    }

    [Test]
    public void Test3() {
      ByteReader reader = new ByteReader(new byte[] { 1, 2, 3, 4, 5, 6, 7, 8, 9 });
      reader = reader.Drop(3).DropWhile(b => b < 5).DropUntil(b => b == 7);
      check(reader, 0, 2, false, 8);
      Assert.AreEqual(true, reader.Next());
      check(reader, 1, 1, false, 9);
      Assert.AreEqual(false, reader.Next());
      Assert.AreEqual(true, reader.AtEnd);
      reader.Reset();
      check(reader, 0, 2, false, 8);
      Assert.AreEqual(true, reader.Next());
      check(reader, 1, 1, false, 9);
      Assert.AreEqual(false, reader.Next());
      Assert.AreEqual(true, reader.AtEnd);
    }

    [Test]
    public void Test4() {
      ByteReader reader = new ByteReader(new byte[] { 1, 2, 3, 4, 5, 6, 7, 8, 9 });
      reader = reader.Take(8).Drop(1).TakeWhile(b => b < 8).DropWhile(b => b < 3).TakeUntil(b => b == 7).DropUntil(b => b == 4);
      check(reader, 0, 3, false, 5);
      Assert.AreEqual(true, reader.Next());
      check(reader, 1, 2, false, 6);
      Assert.AreEqual(true, reader.Next());
      check(reader, 2, 1, false, 7);
      Assert.AreEqual(false, reader.Next());
      Assert.AreEqual(true, reader.AtEnd);
      reader.Reset();
      check(reader, 0, 3, false, 5);
      Assert.AreEqual(true, reader.Next());
      check(reader, 1, 2, false, 6);
      Assert.AreEqual(true, reader.Next());
      check(reader, 2, 1, false, 7);
      Assert.AreEqual(false, reader.Next());
      Assert.AreEqual(true, reader.AtEnd);
    }

    [Test]
    public void Test5() {
      ByteReader reader = new ByteReader(new byte[] { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08 });
      Assert.AreEqual(0x0102, reader.ShortOnBigEndian());
      Assert.AreEqual(0x0403, reader.ShortOnLittleEndian());
      reader.Reset();
      Assert.AreEqual(0x01020304, reader.IntOnBigEndian());
      Assert.AreEqual(0x08070605, reader.IntOnLittleEndian());
    }

    private void check(ByteReader reader, int index, int rest, bool atEnd, byte current) {
      Assert.AreEqual(reader.Index, index);
      Assert.AreEqual(reader.Rest, rest);
      Assert.AreEqual(reader.AtEnd, atEnd);
      Assert.AreEqual(reader.Current, current);
    }
  }
}
