﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;
using San9.Resource.IO;

namespace San9.Resource.Message {
  [TestFixture]
  public class TextMessageTest {
    [TestCase(new byte[] { 0x31, 0x32, 0x05, 0x05, 0x05 }, Result = "12")]
    [TestCase(new byte[] { 0x21, 0x7E, 0x05, 0x05, 0x05 }, Result = "!~")]
    [TestCase(new byte[] { 0xB1, 0xB2, 0xB3, 0x05, 0x05, 0x05 }, Result = "あいう")]
    [TestCase(new byte[] { 0x1B, 0x04B, 0xB1, 0xB2, 0x1B, 0x48, 0xB3, 0x05, 0x05, 0x05 }, Result = "アイう")]
    [TestCase(new byte[] { 0x1B, 0x06B, 0xB1, 0xB2, 0x1B, 0x48, 0xB3, 0x05, 0x05, 0x05 }, Result = "ｱｲう")]
    [TestCase(new byte[] { 0x89, 0x40, 0x9F, 0xFC, 0x05, 0x05, 0x05 }, Result = "院滌")]
    [TestCase(new byte[] { 0x02, 0x01, 0x39, 0x05, 0x05, 0x05 }, Result = "${武将1.番号}")]
    [TestCase(new byte[] { 0x02, 0x8C, 0x31, 0x32, 0x62, 0x05, 0x05, 0x05 }, Result = "#{12}")]
    [TestCase(new byte[] { 0x31, 0x02, 0x8C, 0x33, 0x62, 0x02, 0x01, 0x39, 0xB1, 0x1B, 0x4B, 0xB2, 0x1B, 0x48, 0x1B, 0x6B, 0xB3, 0x1B, 0x48, 0x89, 0x40, 0x05, 0x05, 0x05 }, Result = "1#{3}${武将1.番号}あイｳ院")]
    public string TestCase(byte[] bytes) {
      ByteReader source = new ByteReader(bytes);
      TextMessage message = TextMessage.Parse(source);
      Assert.AreEqual(true, source.AtEnd);
      Assert.AreEqual(bytes, message.Source.ToArray());
      return message.Text;
    }
  }
}
